@@ -249,13 +249,13 @@ public:
     {
         cl_int err;
 
-        err = clEnqueueWriteBuffer(queue_, buffer_.buf_a_, CL_TRUE,
+        err = clEnqueueWriteBuffer(queues_[0], buffer_.buf_a_, CL_TRUE,
                                    buffer_.offA_ * sizeof(T),
                                    buffer_.lda_ * buffer_.a_num_vectors_ *
                                        sizeof(T),
                                    buffer_.a_, 0, NULL, NULL);
 
-        err = clEnqueueWriteBuffer(queue_, buffer_.buf_c_, CL_TRUE,
+        err = clEnqueueWriteBuffer(queues_[0], buffer_.buf_c_, CL_TRUE,
                                    buffer_.offA_ * sizeof(T),
                                    buffer_.ldc_ * buffer_.c_num_vectors_ *
                                        sizeof(T),
@@ -266,7 +266,7 @@ public:
     {
         cl_int err;
 
-        err = clEnqueueWriteBuffer(queue_, buffer_.buf_c_, CL_TRUE,
+        err = clEnqueueWriteBuffer(queues_[0], buffer_.buf_c_, CL_TRUE,
                                    buffer_.offC_ * sizeof(T),
                                    buffer_.ldc_ * buffer_.c_num_vectors_ *
                                        sizeof(T),
@@ -275,7 +275,7 @@ public:
  	void read_gpu_buffer()
 	{
 		cl_int err;
-		err = clEnqueueReadBuffer(queue_, buffer_.buf_c_, CL_TRUE,
+		err = clEnqueueReadBuffer(queues_[0], buffer_.buf_c_, CL_TRUE,
 								  buffer_.offC_*sizeof(T), buffer_.ldc_*buffer_.c_num_vectors_*sizeof(T),
 								  buffer_.c_, 0, NULL, NULL);
 	}
@@ -458,7 +458,7 @@ call_func()
     clblasSsyrk(order_, buffer_.uplo_, buffer_.trans_a_, buffer_.n_,
                      buffer_.k_, buffer_.alpha_, buffer_.buf_a_, buffer_.offA_,
                      buffer_.lda_, buffer_.beta_, buffer_.buf_c_, buffer_.offC_,
-                     buffer_.ldc_, 1, &queue_, 0, NULL, &event_);
+                     buffer_.ldc_, 4, queues_, 0, NULL, &event_);
 
     clWaitForEvents(1, &event_);
     timer.Stop(timer_id);
@@ -484,8 +484,8 @@ xSyrk<float>::roundtrip_func()
 	clblasSsyrk(order_, buffer_.uplo_, buffer_.trans_a_, buffer_.n_,
                      buffer_.k_, buffer_.alpha_, buffer_.buf_a_, buffer_.offA_,
                      buffer_.lda_, buffer_.beta_, buffer_.buf_c_, buffer_.offC_,
-                     buffer_.ldc_, 1, &queue_, 0, NULL, NULL);
-	err = clEnqueueReadBuffer(queue_, buffer_.buf_c_, CL_TRUE,
+                     buffer_.ldc_, numQueues, queues_, 0, NULL, NULL);
+	err = clEnqueueReadBuffer(queues_[0], buffer_.buf_c_, CL_TRUE,
 								  buffer_.offC_*sizeof(float), buffer_.ldc_*buffer_.c_num_vectors_*sizeof(float),
 								  buffer_.c_, 0, NULL, &event_);
 
@@ -503,7 +503,7 @@ call_func()
     clblasDsyrk(order_, buffer_.uplo_, buffer_.trans_a_, buffer_.n_,
                      buffer_.k_, buffer_.alpha_, buffer_.buf_a_, buffer_.offA_,
                      buffer_.lda_, buffer_.beta_, buffer_.buf_c_, buffer_.offC_,
-                     buffer_.ldc_, 1, &queue_, 0, NULL, &event_);
+                     buffer_.ldc_, numQueues, queues_, 0, NULL, &event_);
 
     clWaitForEvents(1, &event_);
     timer.Stop(timer_id);
@@ -529,8 +529,8 @@ xSyrk<double>::roundtrip_func()
 	clblasDsyrk(order_, buffer_.uplo_, buffer_.trans_a_, buffer_.n_,
                      buffer_.k_, buffer_.alpha_, buffer_.buf_a_, buffer_.offA_,
                      buffer_.lda_, buffer_.beta_, buffer_.buf_c_, buffer_.offC_,
-                     buffer_.ldc_, 1, &queue_, 0, NULL, NULL);
-	err = clEnqueueReadBuffer(queue_, buffer_.buf_c_, CL_TRUE,
+                     buffer_.ldc_, numQueues, queues_, 0, NULL, NULL);
+	err = clEnqueueReadBuffer(queues_[0], buffer_.buf_c_, CL_TRUE,
 								  buffer_.offC_*sizeof(double), buffer_.ldc_*buffer_.c_num_vectors_*sizeof(double),
 								  buffer_.c_, 0, NULL, &event_);
 
@@ -548,7 +548,7 @@ call_func()
     clblasCsyrk(order_, buffer_.uplo_, buffer_.trans_a_, buffer_.n_,
                      buffer_.k_, buffer_.alpha_, buffer_.buf_a_, buffer_.offA_,
                      buffer_.lda_, buffer_.beta_, buffer_.buf_c_, buffer_.offC_,
-                     buffer_.ldc_, 1, &queue_, 0, NULL, &event_);
+                     buffer_.ldc_, numQueues, queues_, 0, NULL, &event_);
 
     clWaitForEvents(1, &event_);
     timer.Stop(timer_id);
@@ -574,8 +574,8 @@ xSyrk<cl_float2>::roundtrip_func()
 	clblasCsyrk(order_, buffer_.uplo_, buffer_.trans_a_, buffer_.n_,
                      buffer_.k_, buffer_.alpha_, buffer_.buf_a_, buffer_.offA_,
                      buffer_.lda_, buffer_.beta_, buffer_.buf_c_, buffer_.offC_,
-                     buffer_.ldc_, 1, &queue_, 0, NULL, NULL);
-	err = clEnqueueReadBuffer(queue_, buffer_.buf_c_, CL_TRUE,
+                     buffer_.ldc_, numQueues, queues_, 0, NULL, NULL);
+	err = clEnqueueReadBuffer(queues_[0], buffer_.buf_c_, CL_TRUE,
 								  buffer_.offC_*sizeof(cl_float2), buffer_.ldc_*buffer_.c_num_vectors_*sizeof(cl_float2),
 								  buffer_.c_, 0, NULL, &event_);
 
@@ -606,7 +606,7 @@ call_func()
     clblasZsyrk(order_, buffer_.uplo_, buffer_.trans_a_, buffer_.n_,
                      buffer_.k_, buffer_.alpha_, buffer_.buf_a_, buffer_.offA_,
                      buffer_.lda_, buffer_.beta_, buffer_.buf_c_, buffer_.offC_,
-                     buffer_.ldc_, 1, &queue_, 0, NULL, &event_);
+                     buffer_.ldc_, numQueues, queues_, 0, NULL, &event_);
 
     clWaitForEvents(1, &event_);
     timer.Stop(timer_id);
@@ -632,8 +632,8 @@ xSyrk<cl_double2>::roundtrip_func()
 	clblasZsyrk(order_, buffer_.uplo_, buffer_.trans_a_, buffer_.n_,
                      buffer_.k_, buffer_.alpha_, buffer_.buf_a_, buffer_.offA_,
                      buffer_.lda_, buffer_.beta_, buffer_.buf_c_, buffer_.offC_,
-                     buffer_.ldc_, 1, &queue_, 0, NULL, NULL);
-	err = clEnqueueReadBuffer(queue_, buffer_.buf_c_, CL_TRUE,
+                     buffer_.ldc_, numQueues, queues_, 0, NULL, NULL);
+	err = clEnqueueReadBuffer(queues_[0], buffer_.buf_c_, CL_TRUE,
 								  buffer_.offC_*sizeof(cl_double2), buffer_.ldc_*buffer_.c_num_vectors_*sizeof(cl_double2),
 								  buffer_.c_, 0, NULL, &event_);
 
